<?php
/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

namespace Aheadworks\OneStepCheckout\Block\Adminhtml\Report\CheckoutBehavior\AddressAttributes;

use Aheadworks\OneStepCheckout\Model\Address\Form\AttributeMeta\AvailabilityChecker;
use Aheadworks\OneStepCheckout\Model\Config;
use Magento\Customer\Api\AddressMetadataInterface;
use Magento\Framework\Stdlib\BooleanUtils;

/**
 * Class MetaProvider
 * @package Aheadworks\OneStepCheckout\Block\Adminhtml\Report\CheckoutBehavior\AddressAttributes
 */
class MetaProvider
{
    /**
     * @var AddressMetadataInterface
     */
    private $addressMetadata;

    /**
     * @var AvailabilityChecker
     */
    private $availabilityChecker;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var BooleanUtils
     */
    private $booleanUtils;

    /**
     * @var string
     */
    private $addressType;

    /**
     * @param AddressMetadataInterface $addressMetadata
     * @param AvailabilityChecker $availabilityChecker
     * @param Config $config
     * @param BooleanUtils $booleanUtils
     * @param string $addressType
     */
    public function __construct(
        AddressMetadataInterface $addressMetadata,
        AvailabilityChecker $availabilityChecker,
        Config $config,
        BooleanUtils $booleanUtils,
        $addressType
    ) {
        $this->addressMetadata = $addressMetadata;
        $this->availabilityChecker = $availabilityChecker;
        $this->config = $config;
        $this->booleanUtils = $booleanUtils;
        $this->addressType = $addressType;
    }

    /**
     * Get metadata
     *
     * @return array
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function getMetadata()
    {
        $metadata = [];
        $attrMetadata = $this->addressMetadata->getAttributes('customer_register_address');
        $formConfig = $this->config->getAddressFormConfig($this->addressType);
        foreach ($attrMetadata as $meta) {
            if (!$this->availabilityChecker->isAvailableOnForm($meta)) {
                continue;
            }
            $attributeCode = $meta->getAttributeCode();
            if ($attributeCode == 'region_id') {
                continue;
            }
            if ($meta->getMultilineCount() > 1) {
                for ($line = 0; $line < $meta->getMultilineCount(); $line++) {
                    if (isset($formConfig['attributes'][$attributeCode])) {
                        $customMeta = $formConfig['attributes'][$attributeCode];
                        if ($this->booleanUtils->toBoolean($customMeta[$line]['visible'])) {
                            $metadata[] = [
                                'code' => $attributeCode . '_line_' . $line,
                                'label' => $customMeta[$line]['label'],
                                'required' => $this->booleanUtils->toBoolean($customMeta[$line]['required'])
                            ];
                        }
                    } else {
                        $metadata[] = [
                            'code' => $attributeCode . '_line_' . $line,
                            'label' => $meta->getStoreLabel() . ' Line ' . ($line > 0 ? $line : ''),
                            'required' => $meta->isRequired()
                        ];
                    }
                }
            } else {
                if (isset($formConfig['attributes'][$attributeCode])) {
                    $customMeta = $formConfig['attributes'][$attributeCode];
                    if ($this->booleanUtils->toBoolean($customMeta['visible'])) {
                        $metadata[] = [
                            'code' => $attributeCode,
                            'label' => $customMeta['label'],
                            'required' => $this->booleanUtils->toBoolean($customMeta['required'])
                        ];
                    }
                } else {
                    if ($meta->isVisible()) {
                        $metadata[] = [
                            'code' => $attributeCode,
                            'label' => $meta->getStoreLabel(),
                            'required' => $meta->isRequired()
                        ];
                    }
                }
            }
        }

        return $metadata;
    }
}
