<?php
/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

namespace Aheadworks\OneStepCheckout\Controller\Index;

use Aheadworks\OneStepCheckout\Model\AvailabilityFlag;
use Aheadworks\OneStepCheckout\Model\Config;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Customer\Model\ResourceModel\CustomerRepository;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\View\Result\Page;

/**
 * Class Index
 * @package Aheadworks\OneStepCheckout\Controller\Index
 */
class Index extends Action
{
    /**
     * @var Config
     */
    private $config;

    /**
     * @var AvailabilityFlag
     */
    private $availabilityFlag;

    /**
     * @var CustomerSession
     */
    private $customerSession;

    /**
     * @var CheckoutSession
     */
    private $checkoutSession;

    /**
     * @var CustomerRepository
     */
    private $customerRepository;

    /**
     * @param Context $context
     * @param Config $config
     * @param AvailabilityFlag $availabilityFlag
     * @param CustomerSession $customerSession
     * @param CheckoutSession $checkoutSession
     * @param CustomerRepository $customerRepository
     */
    public function __construct(
        Context $context,
        Config $config,
        AvailabilityFlag $availabilityFlag,
        CustomerSession $customerSession,
        CheckoutSession $checkoutSession,
        CustomerRepository $customerRepository
    ) {
        parent::__construct($context);
        $this->config = $config;
        $this->availabilityFlag = $availabilityFlag;
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->customerRepository = $customerRepository;
    }

    /**
     * {@inheritdoc}
     */
    public function execute()
    {
        if (!$this->availabilityFlag->isAvailable()) {
            $message = $this->availabilityFlag->getMessage();
            if ($message) {
                $this->messageManager->addNoticeMessage(__($message));
            }
            /** @var Redirect $resultRedirect */
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            $resultRedirect->setPath('checkout/cart');
            return $resultRedirect;
        }

        $this->customerSession->regenerateId();
        $this->checkoutSession->setCartWasUpdated(false);

        $customerId = $this->customerSession->getCustomerId();
        if ($customerId) {
            $quote = $this->checkoutSession->getQuote();
            $customer = $this->customerRepository->getById($customerId);
            $quote->assignCustomer($customer);
        }

        /** @var Page $resultPage */
        $resultPage = $this->resultFactory->create(ResultFactory::TYPE_PAGE);
        $pageConfig = $resultPage->getConfig();
        $pageConfig->getTitle()->set($this->config->getCheckoutTitle());
        return $resultPage;
    }
}
