/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

define(
    [
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/model/url-builder',
        'mage/storage',
        'Magento_Checkout/js/model/error-processor',
        'Magento_Checkout/js/model/full-screen-loader',
        'Magento_Customer/js/model/customer',
        'Magento_Checkout/js/model/payment/method-converter',
        'Magento_Checkout/js/model/payment-service',
        'Aheadworks_OneStepCheckout/js/model/editable-item-options',
        'Aheadworks_OneStepCheckout/js/model/image-data'
    ],
    function (
        quote,
        urlBuilder,
        storage,
        errorProcessor,
        fullScreenLoader,
        customer,
        methodConverter,
        paymentService,
        editableItemOptions,
        imageData
    ) {
        'use strict';

        var isAjaxSend = false;

        return function (itemId, options) {
            if (!isAjaxSend) {
                var serviceUrl,
                    payload = {
                        itemId: itemId,
                        options: JSON.stringify(options)
                    };

                if (!customer.isLoggedIn()) {
                    serviceUrl = urlBuilder.createUrl('/awOsc/guest-carts/:cartId/cart-item-options', {
                        cartId: quote.getQuoteId()
                    });
                } else {
                    serviceUrl = urlBuilder.createUrl('/awOsc/carts/mine/cart-item-options', {});
                }
                fullScreenLoader.startLoader();

                isAjaxSend = true;
                return storage.post(
                    serviceUrl,
                    JSON.stringify(payload)
                ).done(
                    function (response) {
                        editableItemOptions.setConfigOptionsData(JSON.parse(response.options_details));
                        imageData.setItemsImageData(JSON.parse(response.image_details));
                        quote.setTotals(response.payment_details.totals);
                        paymentService.setPaymentMethods(methodConverter(response.payment_details.payment_methods));
                    }
                ).fail(
                    function (response) {
                        errorProcessor.process(response);
                    }
                ).always(function () {
                    fullScreenLoader.stopLoader();
                    isAjaxSend = false;
                });
            }
        };
    }
);
