/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

define(
    [
        'ko',
        'underscore'
    ],
    function (ko, _) {
        'use strict';

        var items = ko.observableArray([]);

        return {
            /**
             * Add payment action item
             *
             * @param {string} methodCode
             * @param {Object} item
             */
            add: function (methodCode, item) {
                var existingItem = this.get(methodCode),
                    data = {
                        methodCode: methodCode,
                        item: item
                    };

                if (existingItem) {
                    _.extend(existingItem, data.item);
                } else {
                    items.push(data);
                }
            },

            /**
             * Get payment action item by method code
             *
             * @param {string} methodCode
             * @returns {Object}
             */
            get: function (methodCode) {
                var item =  _.find(items(), function (item) {
                    return item.methodCode == methodCode;
                });

                return typeof item != 'undefined' ? item.item : null;
            },

            /**
             * Get payment action observable
             *
             * @return {Function}
             */
            getItems: function () {
                return items;
            }
        };
    }
);
