/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

define(
    [
        'Magento_Ui/js/lib/view/utils/async',
        'underscore',
        'Aheadworks_OneStepCheckout/js/model/payment-actions',
        'awOscFloatLabel',
        'mage/template',
        'mage/translate'
    ],
    function ($, _, paymentActionList, floatLabel, mageTemplate, $t) {
        'use strict';

        return {
            paymentMethodItemsSelectors: 'div.payment-method',
            actionsToolbarSelector: 'div.actions-toolbar',
            ccMethodCodes: [
                'authorizenet_directpost',
                'braintree',
                'cybersource',
                'eway'
            ],

            /**
             * Process payment method item content
             *
             * @param {Array} elements
             */
            processPaymentMethodContent: function (elements) {
                var methodItem = $(elements).filter(this.paymentMethodItemsSelectors);

                if (methodItem.length > 0) {
                    this._hideActionToolbar(methodItem);
                    this._collectActions(methodItem);
                    this._convertToFloatLabelInputs(methodItem);
                }
            },

            /**
             * Hide action toolbar
             *
             * @param {Object} methodItem
             */
            _hideActionToolbar: function (methodItem) {
                methodItem.find(this.actionsToolbarSelector).hide();
            },

            /**
             * Collect payment actions. Add element to action list
             *
             * @param {Object} methodItem
             */
            _collectActions: function (methodItem) {
                var actionToolbar = methodItem.find(this.actionsToolbarSelector);

                paymentActionList.add(
                    this._getPaymentMethodCode(methodItem),
                    actionToolbar.find('button')
                );
            },

            /**
             * Convert form inputs to inputs with float label
             *
             * @param {Object} methodItem
             */
            _convertToFloatLabelInputs: function (methodItem) {
                var form = methodItem.find('form'),
                    methodCode = this._getPaymentMethodCode(methodItem),
                    expDateContainer = '#' + methodCode + '_cc_type_exp_div';

                if (form.length > 0) {
                    floatLabel({excludeFieldSelector: expDateContainer}, form);
                    if (this._isCCMethod(methodCode, methodItem)) {
                        $.async(
                            expDateContainer,
                            $.proxy(this._processExpirationDateInput, this)
                        );
                    }
                }
            },

            /**
             * Retrieve payment method code from method item element
             *
             * @param {Object} element
             * @returns {string}
             */
            _getPaymentMethodCode: function (element) {
                var input = element.find('input[name="payment[method]"]');

                return input.val();
            },

            /**
             * Check if CC payment method
             *
             * @param {string} methodCode
             * @param {Object} methodItem
             * @returns {boolean}
             */
            _isCCMethod: function (methodCode, methodItem) {
                var form = methodItem.find('form');

                return _.indexOf(methodCode, this.ccMethodCodes) != -1
                    || form.is('#co-transparent-form, #co-transparent-form-braintree');
            },

            /**
             * Process expiration date inputs
             *
             * @param {Object} container
             */
            _processExpirationDateInput: function (container) {
                var expMonth = $(container).find('[name*=cc_exp_month]'),
                    expYear = $(container).find('[name*=cc_exp_year]'),
                    fields = $(container).find('.field');

                $(container).find('label').remove();
                $(container).find('div.no-label').removeClass('no-label');
                if ($(container).hasClass('required')) {
                    fields.addClass('required');
                }
                this._addLabel(expMonth, 'Expiration Month', 'Exp. Month');
                this._addLabel(expYear, 'Expiration Year', 'Exp. Year');

                _.each([expMonth, expYear], function (element) {
                    element.find('option').each(function () {
                        if ($(this).val() == '') {
                            $(this).html('');
                        }
                    })
                });

                fields.each(function () {
                    floatLabel({}, this);
                });
            },

            /**
             * Add label for input element
             *
             * @param {Object} element
             * @param {string} label
             * @param {string} shortLabel
             */
            _addLabel: function (element, label, shortLabel) {
                var labelTemplate = mageTemplate(
                    '<label for="<%- data.for %>" class="label"><span><%- data.label %></span></label>'
                     + '<label for="<%- data.for %>" class="label short"><span><%- data.shortLabel %></span></label>'
                ),
                    target = element.closest('div.control');

                target.before(labelTemplate({
                    data: {
                        for: element.attr('id'),
                        label: $t(label),
                        shortLabel: $t(shortLabel)
                    }
                }));
            }
        };
    }
);
