/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

define(
    [
        'jquery',
        'ko',
        'Magento_Checkout/js/view/summary/abstract-total',
        'Aheadworks_OneStepCheckout/js/model/checkout-data'
    ],
    function ($, ko, Component, checkoutData) {
        'use strict';

        return Component.extend({
            defaults: {
                showForm: false,
                showTotal: false,
                showInitial: true,
                _innerShowTotalValue: false,
                inputValue: '',
                formSelector: ''
            },

            /**
             * @inheritdoc
             */
            initialize: function () {
                var formData;

                this._super();
                this.showTotal.subscribe(function (flag) {
                    this.showForm(!flag);
                    if (flag) {
                        this.inputValue('');
                    }
                }, this);

                formData = checkoutData.getApplicableTotalFormData(this.name);
                if (formData && formData.inputValue) {
                    this.inputValue(formData.inputValue);
                    this.showForm(true);
                }
                this.inputValue.subscribe(function (newValue) {
                    checkoutData.setApplicableTotalFormData(this.name, {'inputValue': newValue});
                }, this);
            },

            /**
             * @inheritdoc
             */
            initObservable: function () {
                this._super();
                this.observe(['showForm', 'inputValue', '_innerShowTotalValue']);

                this.showTotal = ko.pureComputed({

                    /**
                     * Read observable
                     *
                     * @returns {boolean}
                     */
                    read: function () {
                        return this._innerShowTotalValue() && this._getPureTotalValue() != 0;
                    },

                    /**
                     * Write observable
                     *
                     * @param {boolean} flag
                     */
                    write: function (flag) {
                        this._innerShowTotalValue(flag);
                    },

                    owner: this
                });

                this.showInitial = ko.computed(function () {
                    return !this.showForm() && !this.showTotal();
                }, this);

                return this;
            },

            /**
             * On apply button click event handler
             */
            onApplyClick: function () {
                var self = this;

                if (this._validate()) {
                    this._apply().done(function () {
                        self.showTotal(true);
                    });
                }
            },

            /**
             * On cancel button click event handler
             */
            onCancelClick: function () {
                var self = this;

                if (this.showForm()) {
                    this.inputValue('');
                    this._clearValidationErrors();
                    this.showForm(false);
                } else if (this.showTotal() && this._validate()) {
                    this._cancel().done(function () {
                        self.showTotal(false);
                        self.showForm(false);
                    });
                }
            },

            /**
             * Validate form
             */
            _validate: function () {
                var form = $(this.formSelector);

                return form.length > 0
                    && form.validation()
                    && form.validation('isValid');
            },

            /**
             * Clear validation errors
             */
            _clearValidationErrors: function () {
                var form = $(this.formSelector);

                if (form.length > 0 && form.validation()) {
                    form.validation('clearError')
                }
            },

            /**
             * Perform apply action
             *
             * @returns {Deferred}
             */
            _apply: function () {
                return $.Deferred().resolve();
            },

            /**
             * Perform cancel action
             *
             * @returns {Deferred}
             */
            _cancel: function () {
                return $.Deferred().resolve();
            },

            /**
             * Get total value
             *
             * @returns {string}
             */
            getTotalValue: function() {
                return this.getFormattedPrice(this._getPureTotalValue());
            },

            /**
             * Get pure total value amount
             *
             * @returns {number}
             */
            _getPureTotalValue: function () {
                return 0;
            }
        });
    }
);
